<?php 

class OsCustomFieldsHelper {
  
  function __construct(){
  }

  public static function allowed_fields(){
    $allowed_params = array('label',
                            'placeholder',
                            'type',
                            'width',
                            'options',
                            'required',
                            'id');
    return $allowed_params;
  }

  public static function prepare_to_save($array_to_filter){
    // !!TODO
    return $array_to_filter;
  }

	public static function get_custom_field_types(){
		$custom_field_types = [
			'text' => __('Text Field', 'latepoint-custom-fields'),
			'select' => __('Select Box', 'latepoint-custom-fields'),
			'textarea' => __('Text Area Field', 'latepoint-custom-fields'),
			'checkbox' => __('Checkbox', 'latepoint-custom-fields'),
			'google_address_autocomplete' => __('Google Address Autocomplete', 'latepoint-custom-fields'),
			'file_upload' => __('File Upload', 'latepoint-custom-fields')];
		$custom_field_types = apply_filters('latepoint_custom_field_types', $custom_field_types);
		return $custom_field_types;
	}

	public static function load_countries_list($include_empty = true){
  	$country_codes = [
			'' => __('No Restrictions', 'latepoint-custom-fields'),
			'au' => 'Australia',
      'at' => 'Austria',
      'be' => 'Belgium',
      'br' => 'Brazil',
      'ca' => 'Canada',
      'dk' => 'Denmark',
      'ee' => 'Estonia',
      'fi' => 'Finland',
      'fr' => 'France',
      'de' => 'Germany',
      'gr' => 'Greece',
      'hk' => 'Hong Kong',
      'in' => 'India',
      'ie' => 'Ireland',
      'it' => 'Italy',
      'jp' => 'Japan',
      'lv' => 'Latvia',
      'lt' => 'Lithuania',
      'lu' => 'Luxembourg',
      'my' => 'Malaysia',
      'mx' => 'Mexico',
      'nl' => 'Netherlands',
      'nz' => 'New Zealand',
      'no' => 'Norway',
      'pl' => 'Poland',
      'pt' => 'Portugal',
      'ro' => 'Romania',
      'sg' => 'Singapore',
      'sk' => 'Slovakia',
      'si' => 'Slovenia',
      'es' => 'Spain',
      'se' => 'Sweden',
      'ch' => 'Switzerland',
      'ae' => 'United Arab Emirates',
      'gb' => 'United Kingdom',
      'us' => 'United States'];
  	return $country_codes;
	}

  public static function should_step_be_skipped($skip, $step_name, $booking_object){
    if($step_name == 'custom_fields_for_booking'){
      $custom_fields_for_booking = self::get_custom_fields_arr('booking', 'customer', $booking_object);
      if(empty($custom_fields_for_booking)) $skip = true;
    }
    return $skip;
  }

	/**
	 * @param $custom_fields
	 * @param OsModel $model
	 * @param $fields_for //could be 'customer' or 'booking'
	 * @return string
	 */
	public static function output_custom_fields_for_model($custom_fields, OsModel $model, $fields_for = 'customer'){
		$html = '';
		if(empty($custom_fields)) return $html;
    foreach($custom_fields as $custom_field){
      $required_class = ($custom_field['required'] == 'on') ? 'required' : '';
			$name = $fields_for.'[custom_fields]['.$custom_field['id'].']';
			$label = $custom_field['label'];
			$value = $model->get_meta_by_key($custom_field['id'], '');
			$placeholder = !empty($custom_field['placeholder']) ? $custom_field['placeholder'] : '';
			$width = $custom_field['width'];
      switch ($custom_field['type']) {
        case 'text':
          $html.= OsFormHelper::text_field($name, $label, $value, ['class' => $required_class, 'placeholder' => $placeholder], ['class' => $width]);
          break;
        case 'textarea':
          $html.= OsFormHelper::textarea_field($name, $label, $value, ['class' => $required_class, 'placeholder' => $placeholder], ['class' => $width]);
          break;
        case 'select':
          $html.= OsFormHelper::select_field($name, $label, OsFormHelper::generate_select_options_from_custom_field($custom_field['options']), $value, ['class' => $required_class, 'placeholder' => $placeholder], ['class' => $width]);
          break;
        case 'checkbox':
          $html.= OsFormHelper::checkbox_field($name, $label, 'on', ($value == 'on') , ['class' => $required_class], ['class' => $width]);
          break;
        case 'google_address_autocomplete':
          $html.= OsFormHelper::text_field($name, $label, $value, ['class' => $required_class.' latepoint-google-places-autocomplete', 'placeholder' => $placeholder], ['class' => $width]);
          break;
	      case 'file_upload':
					$atts = ['class' => $required_class];
					if($value){
					  $atts['data-route-name'] = OsRouterHelper::build_route_name('custom_fields', 'delete_custom_field_for_model');
					  $atts['data-params'] = OsUtilHelper::build_os_params(['model_id' => $model->id, 'fields_for' => $fields_for, 'custom_field_id' => $custom_field['id']]);
					}
					$html .= OsFormHelper::file_upload_field($name, $label, $value, $atts, ['class' => $width]);
					break;
      }
			$html = apply_filters('latepoint_custom_fields_output_for_model', $html, $custom_field, $model, $fields_for);
    }
		return $html;
	}

  public static function has_validation_errors($custom_field){
  	$errors = [];
  	if(empty($custom_field['label'])) $errors[] = __('Field Label can not be empty', 'latepoint-custom-fields');
  	if(empty($custom_field['type'])){
  		$errors[] = __('Type can not be empty', 'latepoint-custom-fields');
  	}else{
  		if($custom_field['type'] == 'select'){
		  	if(empty($custom_field['options'])) $errors[] = __('Options for select box can not be blank', 'latepoint-custom-fields');
  		}
  	}
  	if(empty($errors)){
  		return false;
  	}else{
  		return $errors;
  	}
  }

  public static function validate_fields($fields_data, $fields_rules, $fields_for){
    $errors = [];
    foreach($fields_rules as $custom_field){
      if($custom_field['required'] != 'on') continue;
			switch($custom_field['type']) {
				case 'checkbox':
	        // checkbox has different "required" validation
	        if (empty($fields_data[$custom_field['id']]) || $fields_data[$custom_field['id']] == 'off') {
		        $error_message = sprintf(__('%s field has to be checked', 'latepoint-custom-fields'), $custom_field['label']);
		        $errors[] = ['type' => 'validation', 'message' => $error_message];
	        }
					break;
				case 'file_upload':
					// file upload fields are only stored in the booking/customer object if they were already saved before (we save the url)
					// so if we don't see them in a model - try get them from file params
					$files = OsParamsHelper::get_file($fields_for);
					if(empty($fields_data[$custom_field['id']]) && empty($files['name']['custom_fields'][$custom_field['id']])){
		        $error_message = sprintf(__('%s has to be attached', 'latepoint-custom-fields'), $custom_field['label']);
		        $errors[] = ['type' => 'validation', 'message' => $error_message];
					}else{
						// TODO check if extension is allowed
						if(!empty($files['name']['custom_fields'][$custom_field['id']])){
							for($i = 0; $i<count($files['name']['custom_fields'][$custom_field['id']]); $i++) {
								$tmp_name = $files['tmp_name']['custom_fields'][$custom_field['id']][$i];
								$name = $files['name']['custom_fields'][$custom_field['id']][$i];
								$wp_filetype = wp_check_filetype_and_ext($tmp_name, $name, false);
								if(empty($wp_filetype['type']) || empty($wp_filetype['ext'])){
					        $error_message = sprintf(__('%s file type is not supported.', 'latepoint-custom-fields'), $custom_field['label']);
					        $errors[] = ['type' => 'validation', 'message' => $error_message];
								}
							}
						}
					}
					break;
				default:
          if(empty($fields_data[$custom_field['id']])){
            $error_message = sprintf( __( '%s can not be blank', 'latepoint-custom-fields' ), $custom_field['label'] );
            $errors[] = ['type' => 'validation', 'message' => $error_message];
          }
					break;
			}
    }
    return $errors;
  }

  public static function save($custom_field, $fields_for = 'customer'){
    $custom_fields = self::get_custom_fields_arr($fields_for, 'all');
    if(!isset($custom_field['id']) || empty($custom_field['id'])){
    	$custom_field['id'] = self::generate_custom_field_id($fields_for);
    }
    $custom_fields[$custom_field['id']] = $custom_field;
    return self::save_custom_fields_arr($custom_fields, $fields_for);
  }

  public static function delete($custom_field_id, $fields_for = 'customer'){
    if(isset($custom_field_id) && !empty($custom_field_id)){
	    $custom_fields = self::get_custom_fields_arr($fields_for, 'all');
	    unset($custom_fields[$custom_field_id]);
	    return self::save_custom_fields_arr($custom_fields, $fields_for);
	  }else{
	  	return false;
	  }
  }

  public static function get_checkbox_value($value){
    return ($value == 'on') ? __('Yes', 'latepoint-custom-fields') : __('No', 'latepoint-custom-fields');
  }

  public static function generate_custom_field_id(){
  	return 'cf_'.OsUtilHelper::random_text('alnum', 8);
  }

  public static function generate_custom_field_condition_id(){
    return 'cfc_'.OsUtilHelper::random_text('alnum', 8);
  }

  public static function generate_condition_form($custom_field_id, $condition_id = false, $condition_data = false){
    $properties = [ 'service' => __('Service', 'latepoint-custom-fields'), 
                    'agent' => __('Agent', 'latepoint-custom-fields')];
    $properties = apply_filters('latepoint_custom_field_condition_properties', $properties);
    // new condition
    if(!$condition_id){
      $condition_id = self::generate_custom_field_condition_id();
      $condition_data = [  'property' => 'service', 
                           'operator' => false, 
                           'value' => false];
    }
    $html = '<div class="cf-condition" data-condition-id="'.$condition_id.'">'.
                '<button class="cf-remove-condition"><i class="latepoint-icon latepoint-icon-cross"></i></button>'.
                OsFormHelper::select_field('custom_fields['.$custom_field_id.'][conditions]['.$condition_id.'][property]', false, $properties, $condition_data['property'], ['class' => 'custom-field-condition-property', 'data-route' => OsRouterHelper::build_route_name('custom_fields', 'available_values_for_condition_property')], ['class' => 'custom-field-condition-property-w']).
                OsFormHelper::select_field('custom_fields['.$custom_field_id.'][conditions]['.$condition_id.'][operator]', false, array('equal' => __('is equal to', 'latepoint-custom-fields'), 'not_equal' => __('is not equal to', 'latepoint-custom-fields')), $condition_data['operator'], [], ['class' => 'custom-field-condition-operator-w']).
                OsFormHelper::multi_select_field('custom_fields['.$custom_field_id.'][conditions]['.$condition_id.'][value]', false, OsFormHelper::model_options_for_multi_select($condition_data['property']), $condition_data['value'] ? explode(',', $condition_data['value']) : [], [], ['class' => 'custom-field-condition-values-w']).
                '<div class="custom-field-condition-property-w" 
                      data-os-action="'.OsRouterHelper::build_route_name('custom_fields', 'new_condition').'" 
                      data-os-params="'.OsUtilHelper::build_os_params(['custom_field_id' => $custom_field_id]).'" 
                      data-os-pass-response="yes" 
                      data-os-pass-this="yes" 
                      data-os-before-after="none" 
                      data-os-after-call="latepointCustomFieldsAdminAddon.add_custom_field_condition"><button class="latepoint-btn-outline latepoint-btn"><i class="latepoint-icon latepoint-icon-plus2"></i><span>'.__('AND', 'latepoint-custom-fields').'</span></button></div>'.
            '</div>';
    return $html;
  }

  // to ignore conditions - pass false for $booking_object
  public static function get_custom_fields_arr($fields_for = 'customer', $visibilityLevel = 'customer', $booking_object = false){
    switch($visibilityLevel){
      case 'all':
        $visibility = ['public', 'admin_agent', 'hidden'];
        break;
      case 'agent':
        $visibility = ['public', 'admin_agent'];
        break;
      case 'customer':
      default:
        $visibility = ['public'];
        break;
    }
    $custom_fields = OsSettingsHelper::get_settings_value('custom_fields_for_'.$fields_for, false);
    if($custom_fields){
      $custom_fields_arr = json_decode($custom_fields, true);
      $visible_fields = [];
      if($custom_fields_arr && is_array($visibility)){
        foreach($custom_fields_arr as $id => $custom_field){
          if(!isset($custom_field['conditions'])) $custom_field['conditions'] = [];
          if(!isset($custom_field['hide_on_summary'])) $custom_field['hide_on_summary'] = 'off';
          if(!isset($custom_field['conditional'])) $custom_field['conditional'] = 'off';
          // if either booking object is not set, or condition when to show is satisfied - show it
          if(!$booking_object || (self::is_condition_satisfied($custom_field, $booking_object)) && (!isset($custom_field['visibility']) || in_array($custom_field['visibility'], $visibility))) $visible_fields[$id] = $custom_field;
        }
      }
	  	return $visible_fields;
    }else{
    	return [];
    }
  }

  public static function is_condition_satisfied($custom_field, $booking_object){
    // no conditions
    if(!$custom_field['conditions'] || $custom_field['conditional'] == 'off') return true;
    foreach($custom_field['conditions'] as $condition){
      $required_values = $condition['value'] ? explode(',', $condition['value']) : [];
      $current_value = false;
      switch($condition['property']){
        case 'service':
          $current_value = $booking_object->service_id;
          break;
        case 'agent':
          $current_value = $booking_object->agent_id;
          break;
      }
      // not satisfied
      if(!in_array($current_value, $required_values)) return false;
    }
    // if reached here - means it is satisfied
    return true;
  }

  public static function save_custom_fields_arr($custom_fields_arr, $fields_for = 'customer'){
    $custom_fields_arr = self::prepare_to_save($custom_fields_arr);
    return OsSettingsHelper::save_setting_by_name('custom_fields_for_'.$fields_for, json_encode($custom_fields_arr));
  }

}