<?php
/**
 * Plugin Name: LatePoint Addon - Google Calendar
 * Plugin URI:  https://latepoint.com/
 * Description: LatePoint addon for google calendar integration
 * Version:     1.2.9
 * Author:      LatePoint
 * Author URI:  https://latepoint.com/
 * Text Domain: latepoint-google-calendar
 * Domain Path: /languages
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly.
}

// If no LatePoint class exists - exit, because LatePoint plugin is required for this addon

if ( ! class_exists( 'LatePointAddonGoogleCalendar' ) ) :

/**
 * Main Addon Class.
 *
 */

class LatePointAddonGoogleCalendar {

  /**
   * Addon version.
   *
   */
  public $version = '1.2.9';
  public $db_version = '1.0.1';
  public $addon_name = 'latepoint-google-calendar';




  /**
   * LatePoint Constructor.
   */
  public function __construct() {
    $this->define_constants();
    $this->init_hooks();
  }

  /**
   * Define LatePoint Constants.
   */
  public function define_constants() {
    $upload_dir = wp_upload_dir();

    global $wpdb;
    $this->define( 'LATEPOINT_TABLE_GCAL_EVENTS', $wpdb->prefix . 'latepoint_gcal_events');
    $this->define( 'LATEPOINT_TABLE_GCAL_RECURRENCES', $wpdb->prefix . 'latepoint_gcal_recurrences');
  }


  public static function public_stylesheets() {
    return plugin_dir_url( __FILE__ ) . 'public/stylesheets/';
  }

  public static function public_javascripts() {
    return plugin_dir_url( __FILE__ ) . 'public/javascripts/';
  }

  public static function images_url(){
    return plugin_dir_url( __FILE__ ) . 'public/images/';
  }

  /**
   * Define constant if not already set.
   *
   */
  public function define( $name, $value ) {
    if ( ! defined( $name ) ) {
      define( $name, $value );
    }
  }

  /**
   * Include required core files used in admin and on the frontend.
   */
  public function includes() {

    // COMPOSER AUTOLOAD
    require (dirname( __FILE__ ) . '/vendor/scoper-autoload.php');

    // CONTROLLERS
    include_once(dirname( __FILE__ ) . '/lib/controllers/google_calendar_controller.php' );

    // HELPERS
    include_once(dirname( __FILE__ ) . '/lib/helpers/google_calendar_helper.php' );

    // MODELS
    include_once(dirname( __FILE__ ) . '/lib/models/google_calendar_event_model.php' );
    include_once(dirname( __FILE__ ) . '/lib/models/gcal_event_recurrence_model.php' );

  }

  public function add_menu_links($menus){
    if(!OsAuthHelper::is_admin_logged_in()) return $menus;
    for($i = 0; $i <= count($menus); $i++){
      if(isset($menus[$i]['id']) && ($menus[$i]['id'] == 'settings')) $menus[$i]['children'][] = array( 'label' => __( 'Google Calendar', 'latepoint-google-calendar' ), 'icon' => '', 'link' => OsRouterHelper::build_link(OsRouterHelper::build_route_name('google_calendar', 'settings')));
    }
    return $menus;
  }

  public function init_hooks(){
    add_action('latepoint_includes', [$this, 'includes']);
    add_action('latepoint_wp_enqueue_scripts', [$this, 'load_front_scripts_and_styles']);
    add_action('latepoint_admin_enqueue_scripts', [$this, 'load_admin_scripts_and_styles']);
    add_action('latepoint_agent_form', [$this, 'agent_form_google_calendar']);
    add_action('latepoint_check_google_cal_watch_channels_refresh', [$this, 'refresh_google_cal_watch_channels']);
    add_action('latepoint_calendar_daily_timeline', [$this, 'daily_timeline'], 10, 2);
    add_action('latepoint_calendar_weekly_timeline', [$this, 'daily_timeline'], 10, 2);
    add_action('latepoint_appointments_timeline', [$this, 'appointments_timeline'], 10, 2);
    add_action('latepoint_booking_updated_admin', [$this, 'process_action_booking_updated'], 10, 2);
    add_action('latepoint_booking_created_admin', [$this, 'process_action_booking_created']);
    add_action('latepoint_booking_updated_frontend', [$this, 'process_action_booking_updated'], 10, 2);
    add_action('latepoint_booking_created_frontend', [$this, 'process_action_booking_created']);
    add_action('latepoint_booking_will_be_deleted', [$this, 'process_action_booking_deleted']);

    add_action('latepoint_after_agent_info_on_index', [$this, 'display_google_connected_for_agent']);

    add_action( 'init', array( $this, 'init' ), 0 );

    add_filter('latepoint_installed_addons', [$this, 'register_addon']);
    add_filter('latepoint_localized_vars_admin', [$this, 'localized_vars_for_admin']);
    add_filter('latepoint_side_menu', [$this, 'add_menu_links']);
    add_filter('latepoint_addons_sqls', [$this, 'db_sqls']);
    add_filter('latepoint_blocked_periods_for_range', [$this, 'insert_events_into_blocked_periods_arr_for_date_range'], 10, 2);

    register_activation_hook(__FILE__, [$this, 'on_activate']);
    register_deactivation_hook(__FILE__, [$this, 'on_deactivate']);


  }

  /**
   * Init LatePoint when WordPress Initialises.
   */
  public function init() {
    // Set up localisation.
    $this->load_plugin_textdomain();
  }

  public function load_plugin_textdomain() {
    load_plugin_textdomain('latepoint-google-calendar', false, dirname(plugin_basename(__FILE__)) . '/languages');
  }

  public function process_action_booking_status_changed($booking_id, $old_status){
    OsGoogleCalendarHelper::create_or_update_booking_in_gcal($booking_id);
  }

  public function process_action_booking_deleted($booking_id){
    OsGoogleCalendarHelper::delete_booking_in_gcal($booking_id);
  }

  public function display_google_connected_for_agent($agent){
    if(OsGoogleCalendarHelper::is_enabled() && OsGoogleCalendarHelper::is_agent_connected_to_gcal($agent->id)){
      echo '<span class="agent-connection-icon"><img title="'.__('Connected to Google Calendar', 'latepoint-google-calendar').'" src="'.LatePoint::images_url().'google-logo-compact.png'.'"/></span>';
    }
  }

  public function process_action_booking_created($booking){
    OsGoogleCalendarHelper::create_or_update_booking_in_gcal($booking->id);
  }

  public function process_action_booking_updated($booking, $old_booking = false){
    // if agent changed - remove the event from old agent
    if($old_booking && $old_booking->agent_id != $booking->agent_id){
      OsGoogleCalendarHelper::remove_booking_from_gcal($booking->id, $old_booking->agent_id);
    }
    OsGoogleCalendarHelper::create_or_update_booking_in_gcal($booking->id);
  }

  public function insert_events_into_blocked_periods_arr($booked_periods_arr, $target_date, $agent_id){
    $events = OsGoogleCalendarHelper::get_events_for_date($target_date, $agent_id);

    if($events){
      foreach($events as $event){
        $booked_periods_arr[] = new \LatePoint\Misc\BlockedPeriod(['start_time' => $event->start_time,
                                              'end_time' => $event->end_time]);
      }
    }
    return $booked_periods_arr;
  }

  public function insert_events_into_blocked_periods_arr_for_date_range($blocked_periods_arr, \LatePoint\Misc\Filter $filter){
    if(!$filter->date_from || !$filter->date_to) return $blocked_periods_arr;
    if(!$filter->connections) return $blocked_periods_arr;

    $date_from_obj = new DateTime( $filter->date_from );
    $date_to_obj   = new DateTime( $filter->date_to );

		$agent_ids = [];
		foreach($filter->connections as $connection){
			$agent_ids[] = $connection->agent_id;
		}
		if($filter->agent_id) $agent_ids[] = $filter->agent_id;
    $agent_ids = array_unique($agent_ids);

    for($day = clone $date_from_obj; $day->format('Y-m-d') <= $date_to_obj->format('Y-m-d'); $day->modify('+1 day')){
			$events = OsGoogleCalendarHelper::get_events_for_date($day->format('Y-m-d'), $agent_ids);
	    if($events){
	      foreach($events as $event){
	        $blocked_periods_arr[$day->format('Y-m-d')][] = new \LatePoint\Misc\BlockedPeriod(['start_time' => $event->start_time,
	                                                                    'end_time' => $event->end_time,
																															        'start_date' => $day->format('Y-m-d'),
																															        'end_date' => $day->format('Y-m-d'),
																															        'agent_id' => $event->agent_id]);
	      }
	    }
    }
    return $blocked_periods_arr;
  }

  public function appointments_timeline($target_date, $args){
    $agent_id = isset($args['agent_id']) ? $args['agent_id'] : false;
    $events = OsGoogleCalendarHelper::get_events_for_date($target_date->format('Y-m-d'), $agent_id);
    if($events){
      foreach($events as $event){
        if(!$args['work_total_minutes']) continue;
        $width = ($event->end_time - $event->start_time) / $args['work_total_minutes'] * 100;
        $left = ($event->start_time - $args['work_start_minutes']) / $args['work_total_minutes'] * 100;
        
        if($width <= 0 || $left >= 100 || (($left + $width) <= 0)) continue;
        if($left < 0){
          $width = $width + $left;
          $left = 0;
        }
        if(($left + $width) > 100) $width = 100 - $left;

        echo '<div class="booking-block gcal-event-booking-block" style="left: '.$left.'%; width: '.$width.'%"><img src="'.LatePoint::images_url().'google-logo-compact.png"/></div>';
      }
    }
  }


  public function daily_timeline($target_date, $args){
    $agent_id = isset($args['agent_id']) ? $args['agent_id'] : false;
    $events = OsGoogleCalendarHelper::get_events_for_date($target_date->format('Y-m-d'), $agent_id);
    if($events){
      foreach($events as $event){
        if($event->start_time >= $args['work_end_minutes'] || $event->end_time <= $args['work_start_minutes']) continue;
        $event_duration = min($event->end_time, $args['work_end_minutes']) - max($event->start_time, $args['work_start_minutes']);
        $event_duration_percent = $event_duration * 100 / $args['work_total_minutes'];
        $event_start_percent = (max($event->start_time,$args['work_start_minutes']) - $args['work_start_minutes']) / ($args['work_end_minutes'] - $args['work_start_minutes']) * 100;
        if($event_start_percent < 0) $event_start_percent = 0;
        if($event_start_percent >= 100) continue;
        ?>
        <div class="ch-day-booking gcal-calendar-event" style="top: <?php echo $event_start_percent; ?>%; height: <?php echo $event_duration_percent; ?>%;">
          <div class="ch-day-booking-i">
            <div class="booking-service-name">
              <img src="<?php echo LatePoint::images_url().'google-logo-compact.png' ?>" alt="">
              <span><?php echo $event->summary; ?></span>  
            </div>
            <div class="booking-time">
	            <?php
	              if(OsGoogleCalendarHelper::is_full_day_event($event->start_time, $event->end_time)){
									_e('Full Day', 'latepoint-google-calendar');
	              }else{
		              echo OsTimeHelper::minutes_to_hours_and_minutes($event->start_time) .' - '. OsTimeHelper::minutes_to_hours_and_minutes($event->end_time);
	              } ?>
            </div>
          </div>
        </div>
        <?php
      }
    }
  }


  public function refresh_google_cal_watch_channels(){
    $agent_meta = new OsAgentMetaModel();
    $all_agents_watch_channels = $agent_meta->where(['meta_key' => 'google_cal_agent_watch_channels'])->get_results_as_models();
    if(!$all_agents_watch_channels) return;
    foreach($all_agents_watch_channels as $agent_watch_channels){

      $watch_channels = json_decode($agent_watch_channels->meta_value, true);
			foreach($watch_channels as $watch_channel){
	      $seconds_left = ($watch_channel['expiration'] / 1000) - time();
	      // less than 10 days before expiration - refresh
	      if($seconds_left < (60*60*24*10)) OsGoogleCalendarHelper::refresh_watch($agent_watch_channels->object_id, $watch_channel['calendar_id']);
			}
    }

  }

  public function on_deactivate(){
    wp_clear_scheduled_hook('latepoint_check_google_cal_watch_channels_refresh');
  }

  public function on_activate(){
    if (! wp_next_scheduled ( 'latepoint_check_google_cal_watch_channels_refresh' )) {
      wp_schedule_event(time(), 'daily', 'latepoint_check_google_cal_watch_channels_refresh');
    }
    if(class_exists('OsDatabaseHelper')) OsDatabaseHelper::check_db_version_for_addons();
    do_action('latepoint_on_addon_activate', $this->addon_name, $this->version);
  }

  public function register_addon($installed_addons){
    $installed_addons[] = ['name' => $this->addon_name, 'db_version' => $this->db_version, 'version' => $this->version];
    return $installed_addons;
  }

  public function db_sqls($sqls){

    global $wpdb;

    $charset_collate = $wpdb->get_charset_collate();

    $sqls[] = "CREATE TABLE ".LATEPOINT_TABLE_GCAL_EVENTS." (
      id int(11) NOT NULL AUTO_INCREMENT,
      summary text,
      start_date date NOT NULL,
      end_date date,
      start_time mediumint(9) NOT NULL,
      end_time mediumint(9),
      agent_id mediumint(9) NOT NULL,
      google_calendar_id text,
      google_event_id text,
      html_link text,
      start_datetime_utc datetime,
      end_datetime_utc datetime,
      created_at datetime,
      updated_at datetime,
      KEY start_date_index (start_date),
      KEY end_date_index (end_date),
      KEY agent_id_index (agent_id),
      PRIMARY KEY  (id)
    ) $charset_collate;";


    $sqls[] = "CREATE TABLE ".LATEPOINT_TABLE_GCAL_RECURRENCES." (
      `id` int(11) NOT NULL AUTO_INCREMENT,
      `until` date,
      `lp_event_id` mediumint(9) NOT NULL,
      `frequency` varchar(30),
      `interval` smallint(5),
      `count` smallint(5),
      `weekday` varchar(30),
      `created_at` datetime,
      `updated_at` datetime,
      KEY lp_event_id_index (lp_event_id),
      KEY frequency_index (frequency),
      PRIMARY KEY  (id)
    ) $charset_collate;";
    return $sqls;
  }



  public function agent_form_google_calendar($agent){
	  if(!$agent->is_new_record() && OsGoogleCalendarHelper::is_enabled()){ ?>
		    <div class="white-box">
		      <div class="white-box-header">
						<div class="os-form-sub-header"><h3><?php _e('Google Calendar Setup', 'latepoint-google-calendar'); ?></h3></div>
		      </div>
		      <div class="white-box-content">
            <?php
            if(OsGoogleCalendarHelper::is_agent_connected_to_gcal($agent->id)) {
							$calendar_id_for_push = OsGoogleCalendarHelper::get_selected_calendar_id_for_push($agent->id);
							$calendar_name_for_push = $calendar_id_for_push ? OsGoogleCalendarHelper::get_calendar_name_by_id($calendar_id_for_push, $agent->id) : false;
							$calendar_ids_for_pull = OsGoogleCalendarHelper::get_selected_calendar_ids_for_pull($agent->id);
							$calendars_for_pull_count = ($calendar_ids_for_pull) ? count(explode(',', $calendar_ids_for_pull)) : 0;
							?>
              <div class="channel-watch-status watch-status-on as-action-list">
	              <div class="channel-action">
	                <div class="status-watch-label">
	                  <i class="latepoint-icon latepoint-icon-checkmark"></i>
	                  <span class="cw-status"><?php echo _e('Access granted', 'latepoint-google-calendar'); ?></span>
			              <a href="#" class="os-google-cal-signout-btn sw-danger"
			                          data-os-prompt="<?php _e('Are you sure you want to disconnect Google Calendar from this agent? All events imported from Google Calendar and all bookings that were added to Google Calendar will be removed.', 'latepoint-google-calendar'); ?>"
			                          data-os-success-action="reload"
			                          data-os-action="<?php echo OsRouterHelper::build_route_name('google_calendar', 'disconnect'); ?>"
			                          data-os-params="<?php echo OsUtilHelper::build_os_params(['agent_id' => $agent->id]) ?>" >
			                (<?php _e('Revoke Access', 'latepoint-google-calendar');?>)
			              </a>
	                </div>
	                <a href="<?php echo OsRouterHelper::build_link(['google_calendar', 'list_bookings_for_sync'], ['agent_id' => $agent->id]); ?>"
	                        class="latepoint-link cw-enable">
	                    <span><?php _e('Open Sync Manager', 'latepoint-google-calendar'); ?></span>
	                    <span class="latepoint-icon latepoint-icon-arrow-right"></span>
	                </a>
	              </div>
	              <div class="channel-action <?php if(!$calendar_name_for_push) echo 'override-status-warning'; ?>">
	                <div class="status-watch-label">
		                <?php if($calendar_name_for_push){ ?>
	                  <i class="latepoint-icon latepoint-icon-checkmark"></i>
	                  <span class="cw-status"><?php echo __('LatePoint bookings will be synced to', 'latepoint-google-calendar'). ' <strong>' . $calendar_name_for_push.'</strong>'; ?></span>
			              <a href="<?php echo OsRouterHelper::build_link(['google_calendar', 'list_bookings_for_sync'], ['agent_id' => $agent->id]); ?>">
			                (<?php _e('Edit', 'latepoint-google-calendar');?>)
			              </a>
										<?php }else{ ?>
		                  <i class="latepoint-icon latepoint-icon-x"></i>
		                  <span class="cw-status"><?php echo __('Calendar to push bookings to was not selected', 'latepoint-google-calendar'); ?></span>
				              <a href="<?php echo OsRouterHelper::build_link(['google_calendar', 'list_bookings_for_sync'], ['agent_id' => $agent->id]); ?>">
				                (<?php _e('Select Calendar', 'latepoint-google-calendar');?>)
				              </a>
										<?php } ?>
	                </div>
	              </div>
	              <div class="channel-action <?php if(!$calendars_for_pull_count) echo 'override-status-warning'; ?>">
	                <div class="status-watch-label">
		                <?php if($calendars_for_pull_count){ ?>
	                  <i class="latepoint-icon latepoint-icon-checkmark"></i>
	                  <span class="cw-status"><?php echo sprintf(__('Events will be loaded from %d connected Google Calendars', 'latepoint-google-calendar'), $calendars_for_pull_count); ?></span>
			              <a href="<?php echo OsRouterHelper::build_link(['google_calendar', 'load_events_for_sync'], ['agent_id' => $agent->id]); ?>">
			                (<?php _e('Edit', 'latepoint-google-calendar');?>)
			              </a>
										<?php }else{ ?>
		                  <i class="latepoint-icon latepoint-icon-x"></i>
		                  <span class="cw-status"><?php echo __('Calendars to pull events from were not selected', 'latepoint-google-calendar'); ?></span>
				              <a href="<?php echo OsRouterHelper::build_link(['google_calendar', 'load_events_for_sync'], ['agent_id' => $agent->id]); ?>">
				                (<?php _e('Select Calendar', 'latepoint-google-calendar');?>)
				              </a>
										<?php } ?>
	                </div>
	              </div>
              </div>
            <?php }else{ ?>
              <div class="channel-watch-status watch-status-off">
                <div class="status-watch-label">
                  <i class="latepoint-icon latepoint-icon-bell-off"></i>
                  <span class="cw-status"><?php _e('Google Calendar is not connected, or access grant has expired. To reconnect, click', 'latepoint-google-calendar'); ?></span>
                </div>
	              <div class="os-google-cal-authorize-btn" id="google-signin-button"
	                   data-agent-id="<?php echo $agent->id; ?>"
                     data-route="<?php echo OsRouterHelper::build_route_name('google_calendar', 'connect'); ?>">
		              <img src="<?php echo LatePointAddonGoogleCalendar::images_url().'google_signin_btn.png'; ?>"/>
	              </div>
              </div>
            <?php } ?>
					</div>
				</div>
	  <?php }
  }

  public function load_front_scripts_and_styles(){

  }

  public function load_admin_scripts_and_styles($localized_vars){
    // Javascripts
    if(OsGoogleCalendarHelper::is_enabled()){
      wp_enqueue_script( 'google-gsi-client', 'https://accounts.google.com/gsi/client', false );
	    wp_enqueue_script( 'latepoint-google-calendar-admin',  $this->public_javascripts() . 'latepoint-google-calendar-admin.js', array('jquery'), $this->version );
    }

    // Stylesheets
    wp_enqueue_style( 'latepoint-google-calendar-admin', $this->public_stylesheets() . 'latepoint-google-calendar-admin.css', false, $this->version );
  }


  public function localized_vars_for_admin($localized_vars){
    // Google Calendar
    if(OsGoogleCalendarHelper::is_enabled()){
      $localized_vars['google_calendar_is_enabled'] = true;
      $localized_vars['google_calendar_client_id'] = OsSettingsHelper::get_settings_value('google_calendar_client_id');
    }else{
      $localized_vars['google_calendar_is_enabled'] = false;
    }
    return $localized_vars;
  }

}

endif;

if ( in_array( 'latepoint/latepoint.php', get_option( 'active_plugins', array() ) )  || array_key_exists('latepoint/latepoint.php', get_site_option('active_sitewide_plugins', array())) ) {
	$LATEPOINT_ADDON_GOOGLE_CALENDAR = new LatePointAddonGoogleCalendar();
}
$latepoint_session_salt = 'ZjYwNTU0MzgtNDAwNC00OTgzLTgzMTUtNWZmNDlkNmUwNjUy';
