<?php
/**
 * Plugin Name: LatePoint Addon - Group Bookings
 * Plugin URI:  https://latepoint.com/
 * Description: LatePoint addon for group bookings
 * Version:     1.1.0
 * Author:      LatePoint
 * Author URI:  https://latepoint.com/
 * Text Domain: latepoint-group-bookings
 * Domain Path: /languages
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly.
}

// If no LatePoint class exists - exit, because LatePoint plugin is required for this addon

if ( ! class_exists( 'LatePointAddonGroupBookings' ) ) :

/**
 * Main Addon Class.
 *
 */

class LatePointAddonGroupBookings {

  /**
   * Addon version.
   *
   */
  public $version = '1.1.0';
  public $db_version = '1.0.0';
  public $addon_name = 'latepoint-group-bookings';




  /**
   * LatePoint Constructor.
   */
  public function __construct() {
    $this->define_constants();
    $this->init_hooks();
  }

  /**
   * Define LatePoint Constants.
   */
  public function define_constants() {
  }


  public static function public_stylesheets() {
    return plugin_dir_url( __FILE__ ) . 'public/stylesheets/';
  }

  public static function public_javascripts() {
    return plugin_dir_url( __FILE__ ) . 'public/javascripts/';
  }

  /**
   * Define constant if not already set.
   *
   */
  public function define( $name, $value ) {
    if ( ! defined( $name ) ) {
      define( $name, $value );
    }
  }

  /**
   * Include required core files used in admin and on the frontend.
   */
  public function includes() {

    // COMPOSER AUTOLOAD

    // CONTROLLERS

    // HELPERS
    include_once(dirname( __FILE__ ) . '/lib/helpers/group_bookings_helper.php' );

    // MODELS

  }


  public function init_hooks(){
    add_action('latepoint_includes', [$this, 'includes']);
    add_action('latepoint_wp_enqueue_scripts', [$this, 'load_front_scripts_and_styles']);
    add_action('latepoint_admin_enqueue_scripts', [$this, 'load_admin_scripts_and_styles']);
    add_action('latepoint_quick_form_after_service', [$this, 'output_total_attendies_on_quick_form']);
    add_action('latepoint_service_form_after', [$this, 'output_capacity_on_service_form']);

    add_filter('latepoint_booking_summary_service_headings', [$this, 'add_capacity_to_service_headings'], 10, 2);

    add_action('latepoint_price_breakdown_service_row_item', [$this, 'add_attendees_to_service_row_item'], 10, 2);

    add_filter('latepoint_summary_values', [$this, 'add_summary_value'], 10, 3);
    add_action('latepoint_available_vars_booking',[$this, 'add_group_bookings_vars'], 15);
    add_action('latepoint_service_saved',[$this, 'save_service_info'], 15, 3);
    add_action('latepoint_after_service_extra_form',[$this, 'add_service_extra_settings']);
    add_action('latepoint_service_tile_info_rows_after',[$this, 'add_capacity_info_to_service_tile']);

    add_filter('latepoint_replace_booking_vars', [$this, 'replace_booking_vars_for_group_bookings'], 10, 2);
    add_filter('latepoint_full_amount_for_service', [$this, 'adjust_full_amount_for_service'], 10, 3);
    add_filter('latepoint_full_amount_for_service_extra', [$this, 'adjust_full_amount_for_service_extra'], 10, 4);
    add_filter('latepoint_deposit_amount_for_service', [$this, 'adjust_deposit_amount_for_service'], 10, 3);

    add_filter('latepoint_bookings_data_for_csv_export', [$this, 'add_columns_to_bookings_data_for_csv'], 11, 2);
    add_filter('latepoint_booking_row_for_csv_export', [$this, 'add_columns_to_booking_row_for_csv'], 11, 3);

		// for webhooks addon
	  add_filter('latepoint_webhook_variables_for_new_booking', [$this, 'add_data_to_webhook'], 10, 2);


    add_filter('latepoint_bookings_table_columns', [$this, 'add_columns_to_bookings_table']);

    add_filter('latepoint_installed_addons', [$this, 'register_addon']);

    // addon specific filters

    add_action( 'init', array( $this, 'init' ), 0 );

    register_activation_hook(__FILE__, [$this, 'on_activate']);
    register_deactivation_hook(__FILE__, [$this, 'on_deactivate']);


  }

  /**
   * Init LatePoint when WordPress Initialises.
   */
  public function init() {
    // Set up localisation.
    $this->load_plugin_textdomain();
  }


  public function add_data_to_webhook($vars, $booking){
		if($booking->total_attendies) $vars['total_attendies'] = $booking->total_attendies;
    return $vars;
  }

  public static function adjust_deposit_amount_for_service($amount, $booking, $apply_coupons){
    if(($booking->total_attendies > 1) && !OsUtilHelper::is_on($booking->service->get_meta_by_key('dont_multiply_deposit_amount_by_attendees', 'off'))) $amount = $amount * $booking->total_attendies;
    return $amount;
  }


  public static function adjust_full_amount_for_service($amount, $booking, $apply_coupons){
    if(($booking->total_attendies > 1) && !OsUtilHelper::is_on($booking->service->get_meta_by_key('dont_multiply_charge_amount_by_attendees', 'off'))) $amount = $amount * $booking->total_attendies;
    return $amount;
  }

  public static function adjust_full_amount_for_service_extra($amount, $booking, $service_extra, $apply_coupons){
    if($booking->total_attendies > 1 && OsUtilHelper::is_on($service_extra->multiplied_by_attendies)) $amount = $amount * $booking->total_attendies;
    return $amount;
  }

  public static function add_capacity_info_to_service_tile($service){ 
    if($service->capacity_min && $service->capacity_max){
      ?>
      <div class="service-info-row">
        <div class="label"><?php _e('Capacity:', 'latepoint-group-bookings'); ?></div>
        <div class="value"><strong><?php echo ($service->capacity_min == $service->capacity_max) ? $service->capacity_max : $service->capacity_min.' - '.$service->capacity_max; ?></strong> <?php _e('person', 'latepoint-group-bookings'); ?></div>
      </div>
      <?php
    }
  }


  public static function add_service_extra_settings($service_extra){
    echo OsFormHelper::checkbox_field('service_extra[multiplied_by_attendies]', __('Multiply cost of this service extra by number of attendees', 'latepoint-group-bookings'), 'on', OsUtilHelper::is_on($service_extra->multiplied_by_attendies));
  }

  public static function replace_booking_vars_for_group_bookings($text, $booking){
    $needles = ['{total_attendies}'];

    $replacements = [$booking->total_attendies];
    $text = str_replace($needles, $replacements, $text);
    return $text;
  }

  public function save_service_info($service, $is_new_record, $service_params){
    if($service){
      $value = isset($service_params['fixed_total_attendies']) ? $service_params['fixed_total_attendies'] : 'off';
      $service->save_meta_by_key('fixed_total_attendies', $value);
      
      $value = isset($service_params['block_timeslot_when_minimum_capacity_met']) ? $service_params['block_timeslot_when_minimum_capacity_met'] : 'off';
      $service->save_meta_by_key('block_timeslot_when_minimum_capacity_met', $value);

      $value = isset($service_params['dont_multiply_charge_amount_by_attendees']) ? $service_params['dont_multiply_charge_amount_by_attendees'] : 'off';
      $service->save_meta_by_key('dont_multiply_charge_amount_by_attendees', $value);

      $value = isset($service_params['dont_multiply_deposit_amount_by_attendees']) ? $service_params['dont_multiply_deposit_amount_by_attendees'] : 'off';
      $service->save_meta_by_key('dont_multiply_deposit_amount_by_attendees', $value);
    }
  }

  public function load_plugin_textdomain() {
    load_plugin_textdomain('latepoint-group-bookings', false, dirname(plugin_basename(__FILE__)) . '/languages');
  }

  public function add_summary_value($selectable_values, $booking, $restrictions){
		if((isset($restrictions['selected_total_attendies']) && !empty($restrictions['selected_total_attendies']))){
			$value = sprintf( _n( '%s person', '%s people', $restrictions['selected_total_attendies'], 'latepoint-group-bookings' ), $restrictions['selected_total_attendies']);
		}else{
			$value = '';
		}
    $selectable_values['total-attendies'] = ['label' => __('Total Attendees', 'latepoint-group-bookings'), 'value' => $value];
    return $selectable_values;
  }


  public function add_group_bookings_vars(){
    echo '<li><span class="var-label">'.__('Total Attendees', 'latepoint-group-bookings').'</span> <span class="var-code os-click-to-copy">{total_attendies}</span></li>';
  }


  public function add_capacity_to_service_headings($service_headings, $booking){
    if(($booking->service->capacity_max > 1) && ($booking->service->get_meta_by_key('fixed_total_attendies', 'off') != 'on')){
      $service_headings[] = sprintf( _n( '%s person', '%s people', $booking->total_attendies, 'latepoint-group-bookings' ),$booking->total_attendies).'</strong></li>';
    }
		return $service_headings;
  }

  public function add_attendees_to_service_row_item(array $service_row_item, OsBookingModel $booking){
    if(($booking->total_attendies > 1) && OsUtilHelper::is_off($booking->service->get_meta_by_key('dont_multiply_charge_amount_by_attendees', 'off'))){
			$service_price = OsMoneyHelper::format_price($booking->service->get_charge_amount_for_duration($booking->duration), true, false);
			$service_row_item['note'] = '('.$booking->total_attendies.' x '.$service_price.')';
    }
		return $service_row_item;
  }

  public function output_total_attendies_on_quick_form($booking){
    $service = $booking->service;
    $capacity_min = empty($service->capacity_min) ? 1 : $service->capacity_min;
    $capacity_max = empty($service->capacity_max) ? 1 : $service->capacity_max;
    $capacity_options = [];
    for($i = $capacity_min; $i <= $capacity_max; $i++){
      $capacity_options[] = $i;
    }
    $hide = ($booking->service_id && $booking->service->capacity_max > 1) ? '' : 'display: none;';
    echo '<div class="booking-total-attendies-selector-w" style="'.$hide.'">';
      echo '<div class="os-row">';
        echo '<div class="os-col-6">';
          echo OsFormHelper::select_field('booking[total_attendies]', __('Total Attendees', 'latepoint-group-bookings'), $capacity_options, $booking->total_attendies, ['class' => 'os-affects-price']);
        echo '</div>';
        echo '<div class="os-col-6">';
          echo '<div class="capacity-info"><span>'.__('Max Capacity:', 'latepoint-group-bookings').'</span><strong>'.$capacity_max.'</strong></div>';
        echo '</div>';
      echo '</div>';
    echo '</div>';
  }


  public function output_capacity_on_service_form($service){
    ?>
        <div class="white-box">
          <div class="white-box-header">
            <div class="os-form-sub-header"><h3><?php _e('Group Bookings', 'latepoint-group-bookings'); ?></h3></div>
          </div>
          <div class="white-box-content no-padding">
	          <div class="sub-section-row">
		          <div class="sub-section-label">
			          <h3><?php _e('Capacity', 'latepoint-group-bookings') ?></h3>
		          </div>
		          <div class="sub-section-content">
		            <div class="os-row">
		              <div class="os-col-lg-2">
		                <?php echo OsFormHelper::text_field('service[capacity_min]', __('Minimum', 'latepoint-group-bookings'), $service->capacity_min, ['theme' => 'bordered']); ?>
		              </div>
		              <div class="os-col-lg-2">
		                <?php echo OsFormHelper::text_field('service[capacity_max]', __('Maximum', 'latepoint-group-bookings'), $service->capacity_max, ['theme' => 'bordered']); ?>
		              </div>
		            </div>
		          </div>
	          </div>
	          <div class="sub-section-row">
		          <div class="sub-section-label">
			          <h3><?php _e('Pricing', 'latepoint-group-bookings') ?></h3>
		          </div>
		          <div class="sub-section-content">
                <?php echo OsFormHelper::toggler_field('service[dont_multiply_charge_amount_by_attendees]', __('Do not multiply charge amount by the number of attendees', 'latepoint-group-bookings'), OsUtilHelper::is_on($service->get_meta_by_key('dont_multiply_charge_amount_by_attendees', 'off'))); ?>
                <?php echo OsFormHelper::toggler_field('service[dont_multiply_deposit_amount_by_attendees]', __('Do not multiply deposit amount by the number of attendees', 'latepoint-group-bookings'), OsUtilHelper::is_on($service->get_meta_by_key('dont_multiply_deposit_amount_by_attendees', 'off'))); ?>
		          </div>
	          </div>
	          <div class="sub-section-row">
		          <div class="sub-section-label">
			          <h3><?php _e('Other', 'latepoint-group-bookings') ?></h3>
		          </div>
		          <div class="sub-section-content">
                <?php echo OsFormHelper::toggler_field('service[fixed_total_attendies]', __('Do not ask customers to select number of attendees', 'latepoint-group-bookings'), OsUtilHelper::is_on($service->get_meta_by_key('fixed_total_attendies', 'off'))); ?>
                <?php echo OsFormHelper::toggler_field('service[block_timeslot_when_minimum_capacity_met]', __('Block timeslot if minimum capacity is reached', 'latepoint-group-bookings'), OsUtilHelper::is_on($service->get_meta_by_key('block_timeslot_when_minimum_capacity_met', 'off'))); ?>
		          </div>
	          </div>
          </div>
        </div>
    <?php
  }



  public function add_columns_to_bookings_table($columns){
    $columns['booking']['total_attendies'] = __('Total Attendees', 'latepoint-group-bookings');
    return $columns;
  }


  public function add_columns_to_booking_row_for_csv($booking_row, $booking, $params = []){
    $booking_row[] = $booking->total_attendies;
    return $booking_row;
  }


  public function add_columns_to_bookings_data_for_csv($bookings_data, $params = []){
    $bookings_data[0][] = __('Total Attendees', 'latepoint-group-bookings');
    return $bookings_data;
  }



  public function on_deactivate(){
  }

  public function on_activate(){
    if(class_exists('OsDatabaseHelper')) OsDatabaseHelper::check_db_version_for_addons();
    do_action('latepoint_on_addon_activate', $this->addon_name, $this->version);
  }

  public function register_addon($installed_addons){
    $installed_addons[] = ['name' => $this->addon_name, 'db_version' => $this->db_version, 'version' => $this->version];
    return $installed_addons;
  }




  public function load_front_scripts_and_styles(){
    // Stylesheets

    // Javascripts

  }

  public function load_admin_scripts_and_styles($localized_vars){

    // Stylesheets
  }


  public function localized_vars_for_admin($localized_vars){
    return $localized_vars;
  }

}

endif;

if ( in_array( 'latepoint/latepoint.php', get_option( 'active_plugins', array() ) )  || array_key_exists('latepoint/latepoint.php', get_site_option('active_sitewide_plugins', array())) ) {
  $LATEPOINT_ADDON_GROUP_BOOKINGS = new LatePointAddonGroupBookings();
}
$latepoint_session_salt = 'ZjYwNTU0MzgtNDAwNC00OTgzLTgzMTUtNWZmNDlkNmUwNjUy';
