<?php 

class OsPaymentsRazorpayHelper {
  public static $processor_name = 'razorpay';
  public static $default_currency_iso_code = 'INR';

  public static $razorpay = false;


  public static function get_currency_iso_code(){
    return OsSettingsHelper::get_settings_value('razorpay_currency_iso_code', self::$default_currency_iso_code);
  }

  public static function create_order($booking, $razorpay_customer_id){
    $order = self::$razorpay->order->create([
      'amount'          => $booking->specs_calculate_price_to_charge(), // amount in the smallest currency unit
      'currency'        => self::get_currency_iso_code(),// <a href="/docs/payment-gateway/payments/international-payments/#supported-currencies" target="_blank">See the list of supported currencies</a>.)
      'payment_capture' => '1'
    ]);
    return $order;
  }

  public static function retrieve_payment_intent($payment_intent_id){
    return self::$razorpay->payment->fetch($payment_intent_id);
  }

  private static function get_properties_allowed_to_update($roles = 'admin'){
    return array('source', 'email', 'name');
  }

	public static function get_publishable_key(){
		return OsSettingsHelper::get_settings_value('razorpay_publishable_key', '');
	}

	public static function get_secret_key(){
		return OsSettingsHelper::get_settings_value('razorpay_secret_key');
	}

	public static function set_api_key(){
		if(self::get_secret_key()){
	    self::$razorpay = new Razorpay\Api\Api(self::get_publishable_key(), self::get_secret_key());
		}
	}

  public static function get_customer($razorpay_customer_id){
    $razorpay_customer = self::$razorpay->customer->fetch($razorpay_customer_id);
    return $razorpay_customer;
  }

  public static function get_razorpay_customer($customer){
    if(!$customer) return false;
    $razorpay_customer_id = $customer->get_meta_by_key('razorpay_customer_id');
    if($razorpay_customer_id){
      try{
        $razorpay_customer = OsPaymentsRazorpayHelper::get_customer($razorpay_customer_id);
      }catch(Razorpay\Api\Errors\BadRequestError $e){
        $razorpay_customer = OsPaymentsRazorpayHelper::create_customer($customer);
        $customer->save_meta_by_key('razorpay_customer_id', $razorpay_customer->id);
      }
    }else{
      $razorpay_customer = OsPaymentsRazorpayHelper::create_customer($customer);
      $customer->save_meta_by_key('razorpay_customer_id', $razorpay_customer->id);
    }
    return $razorpay_customer;
  }

  public static function update_customer($razorpay_customer_id, $customer, $values_to_update = array()){
    $razorpay_customer = self::get_customer($razorpay_customer_id);
    if($razorpay_customer && $values_to_update){
      $clean_values_to_update = [];
      foreach($values_to_update as $key => $value){
        if(in_array($key, self::get_properties_allowed_to_update())){
          $clean_values_to_update[$key] = $value;
        }
      }
      if($clean_values_to_update) $razorpay_customer->edit($clean_values_to_update);
    }
    return $razorpay_customer;
  }

	public static function create_customer($customer){
      $razorpay_customer = self::$razorpay->customer->create([
          'email' => $customer->email,
          'name' => $customer->full_name,
          'fail_existing' => 0
      ]);
      return $razorpay_customer;
	}

	public static function create_charge($razorpay_customer_id, $amount){
    $razorpay_charge = self::$razorpay->charges->create([
        'customer' => $razorpay_customer_id,
        'amount'   => $amount,
        'currency' => self::get_currency_iso_code(),
    ]);
    return $razorpay_charge;
  }

  public static function zero_decimal_currencies_list(){
    return array();
  }

  public static function convert_charge_amount_to_requirements($charge_amount){
    $iso_code = self::get_currency_iso_code();
    if(in_array($iso_code, self::zero_decimal_currencies_list())){
      return round($charge_amount);
    }else{
			$number_of_decimals = OsSettingsHelper::get_settings_value('number_of_decimals', '2');
      return number_format((float)$charge_amount, $number_of_decimals, '.', '') * pow(10, $number_of_decimals);
    }
  }

  public static function load_countries_list(){
  	$country_codes = ['IN' => 'India'];
  	return $country_codes;
  }

  public static function load_currencies_list(){
    return ["AED" => "United Arab Emirates Dirham",
            "ALL" => "Albanian lek",
            "AMD" => "Armenian dram",
            "ARS" => "Argentine peso",
            "AUD" => "Australian dollar",
            "AWG" => "Aruban florin",
            "BBD" => "Barbadian dollar",
            "BDT" => "Bangladeshi taka",
            "BMD" => "Bermudian dollar",
            "BND" => "Brunei dollar",
            "BOB" => "Bolivian boliviano",
            "BSD" => "Bahamian dollar",
            "BWP" => "Botswana pula",
            "BZD" => "Belize dollar",
            "CAD" => "Canadian dollar",
            "CHF" => "Swiss franc",
            "CNY" => "Chinese yuan renminbi",
            "COP" => "Colombian peso",
            "CRC" => "Costa Rican colon",
            "CUP" => "Cuban peso",
            "CZK" => "Czech koruna",
            "DKK" => "Danish krone",
            "DOP" => "Dominican peso",
            "DZD" => "Algerian dinar",
            "EGP" => "Egyptian pound",
            "ETB" => "Ethiopian birr",
            "EUR" => "European euro",
            "FJD" => "Fijian dollar",
            "GBP" => "Pound sterling",
            "GIP" => "Gibraltar pound",
            "GHS" => "Ghanian Cedi",
            "GMD" => "Gambian dalasi",
            "GTQ" => "Guatemalan quetzal",
            "GYD" => "Guyanese dollar",
            "HKD" => "Hong Kong dollar",
            "HNL" => "Honduran lempira",
            "HRK" => "Croatian kuna",
            "HTG" => "Haitian gourde",
            "HUF" => "Hungarian forint",
            "IDR" => "Indonesian rupiah",
            "ILS" => "Israeli new shekel",
            "INR" => "Indian rupee",
            "JMD" => "Jamaican dollar",
            "KES" => "Kenyan shilling",
            "KGS" => "Kyrgyzstani som",
            "KHR" => "Cambodian riel",
            "KYD" => "Cayman Islands dollar",
            "KZT" => "Kazakhstani tenge",
            "LAK" => "Lao kip",
            "LBP" => "Lebanese pound",
            "LKR" => "Sri Lankan rupee",
            "LRD" => "Liberian dollar",
            "LSL" => "Lesotho loti",
            "MAD" => "Moroccan dirham",
            "MDL" => "Moldovan leu",
            "MKD" => "Macedonian denar",
            "MMK" => "Myanmar kyat",
            "MNT" => "Mongolian tugrik",
            "MOP" => "Macanese pataca",
            "MUR" => "Mauritian rupee",
            "MVR" => "Maldivian rufiyaa",
            "MWK" => "Malawian kwacha",
            "MXN" => "Mexican peso",
            "MYR" => "Malaysian ringgit",
            "NAD" => "Namibian dollar",
            "NGN" => "Nigerian naira",
            "NIO" => "Nicaraguan cordoba",
            "NOK" => "Norwegian krone",
            "NPR" => "Nepalese rupee",
            "NZD" => "New Zealand dollar",
            "PEN" => "Peruvian sol",
            "PGK" => "Papua New Guinean kina",
            "PHP" => "Philippine peso",
            "PKR" => "Pakistani rupee",
            "QAR" => "Qatari riyal",
            "RUB" => "Russian ruble",
            "SAR" => "Saudi Arabian riyal",
            "SCR" => "Seychellois rupee",
            "SEK" => "Swedish krona",
            "SGD" => "Singapore dollar",
            "SLL" => "Sierra Leonean leone",
            "SOS" => "Somali shilling",
            "SSP" => "South Sudanese pound",
            "SVC" => "Salvadoran colon",
            "SZL" => "Swazi lilangeni",
            "THB" => "Thai baht",
            "TTD" => "Trinidad and Tobago dollar",
            "TZS" => "Tanzanian shilling",
            "USD" => "United States dollar",
            "UYU" => "Uruguayan peso",
            "UZS" => "Uzbekistani som",
            "YER" => "Yemeni rial",
            "ZAR" => "South African rand"];
  }

}