<?php
/**
 * Plugin Name: LatePoint Addon - Payments Squareup
 * Plugin URI:  https://latepoint.com/
 * Description: LatePoint addon for payments via Squareup
 * Version:     1.0.2
 * Author:      LatePoint
 * Author URI:  https://latepoint.com/
 * Text Domain: latepoint-payments-squareup
 * Domain Path: /languages
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly.
}

// If no LatePoint class exists - exit, because LatePoint plugin is required for this addon

if ( ! class_exists( 'LatePointPaymentsSquareup' ) ) :

/**
 * Main Addon Class.
 *
 */

class LatePointPaymentsSquareup {

  /**
   * Addon version.
   *
   */
  public $version = '1.0.2';
  public $db_version = '1.0.0';
  public $addon_name = 'latepoint-payments-squareup';

  public $processor_code = 'squareup';



  /**
   * LatePoint Constructor.
   */
  public function __construct() {
    $this->define_constants();
    $this->init_hooks();
  }

  /**
   * Define LatePoint Constants.
   */
  public function define_constants() {
  }


  public static function public_stylesheets() {
    return plugin_dir_url( __FILE__ ) . 'public/stylesheets/';
  }

  public static function public_javascripts() {
    return plugin_dir_url( __FILE__ ) . 'public/javascripts/';
  }

  public static function images_url() {
    return plugin_dir_url( __FILE__ ) . 'public/images/';
  }

  /**
   * Define constant if not already set.
   *
   */
  public function define( $name, $value ) {
    if ( ! defined( $name ) ) {
      define( $name, $value );
    }
  }

  /**
   * Include required core files used in admin and on the frontend.
   */
  public function includes() {

    // COMPOSER AUTOLOAD
    require (dirname( __FILE__ ) . '/vendor/autoload.php');

    // CONTROLLERS
    include_once( dirname( __FILE__ ) . '/lib/controllers/payments_squareup_controller.php' );

    // HELPERS
    include_once( dirname( __FILE__ ) . '/lib/helpers/payments_squareup_helper.php' );

    // MODELS

  }


  public function init_hooks(){
    add_action('latepoint_init', [$this, 'latepoint_init']);
    add_action('latepoint_includes', [$this, 'includes']);
    add_action('latepoint_admin_enqueue_scripts', [$this, 'load_admin_scripts_and_styles']);
    add_action('latepoint_payment_processor_settings',[$this, 'add_settings_fields'], 10);

    add_filter('latepoint_payment_processors', [$this, 'register_payment_processor'], 10, 2);
    add_filter('latepoint_all_payment_methods', [$this, 'register_payment_methods']);
    add_filter('latepoint_enabled_payment_methods', [$this, 'register_enabled_payment_methods']);
    add_filter('latepoint_installed_addons', [$this, 'register_addon']);

    add_filter('latepoint_localized_vars_front', [$this, 'localized_vars_for_front']);
    add_filter('latepoint_localized_vars_admin', [$this, 'localized_vars_for_admin']);

    add_filter('latepoint_convert_charge_amount_to_requirements', [$this, 'convert_charge_amount_to_requirements'], 10, 2);


    add_action('latepoint_wp_enqueue_scripts', [$this, 'load_front_scripts_and_styles']);
    add_filter('latepoint_prepare_step_vars_for_view', [$this, 'add_vars_for_payment_step'], 10, 3);
    add_filter('latepoint_prepare_step_booking_object', [$this, 'prepare_booking_object_for_step'], 10, 2);
    add_filter('latepoint_payment_sub_step_for_payment_step', [$this, 'sub_step_for_payment_step']);

    add_action('latepoint_payment_step_content',[$this, 'output_payment_step_contents'], 10, 2);
    add_filter('latepoint_process_payment_for_booking', [$this, 'process_payment'], 10, 3);
    add_filter('latepoint_encrypted_settings', [$this, 'add_encrypted_settings']);


    // addon specific filters

    add_action( 'init', array( $this, 'init' ), 0 );

    register_activation_hook(__FILE__, [$this, 'on_activate']);
    register_deactivation_hook(__FILE__, [$this, 'on_deactivate']);
  }


  public function add_encrypted_settings($encrypted_settings){
    $encrypted_settings[] = 'squareup_secret_key';
    $encrypted_settings[] = 'squareup_webhook_secret';
    return $encrypted_settings;
  }


  public function process_payment($result, $booking, $customer){
    if(OsPaymentsHelper::is_payment_processor_enabled($this->processor_code)){
      switch($booking->payment_method){
        case 'card':
          if($booking->payment_token){
            $squareup_transaction = OsPaymentsSquareupHelper::charge_by_token($booking->payment_token, $booking, $customer);
            if($squareup_transaction['status'] == LATEPOINT_STATUS_SUCCESS){
              $result['processor'] = $this->processor_code;
              $result['status'] = LATEPOINT_STATUS_SUCCESS;
              $result['charge_id'] = $squareup_transaction['charge_id'];
              $result['funds_status'] = $squareup_transaction['funds_status'];
            }else{
              $result['status'] = LATEPOINT_STATUS_ERROR;
              $result['message'] = $squareup_transaction['message'];
              $booking->add_error('send_to_step', $result['message'], 'payment');
            }
          }else{
            $result['status'] = LATEPOINT_STATUS_ERROR;
            $result['message'] = __('Payment Error 23JDF38', 'latepoint-payments-squareup');
            $booking->add_error('payment_error', $result['message']);
          }
        break;
      }
    }
    return $result;
  }


  public function convert_charge_amount_to_requirements($charge_amount, $payment_method){
    if(OsPaymentsHelper::is_payment_processor_enabled($this->processor_code)){
      if(in_array($payment_method, array_keys($this->get_supported_payment_methods()))){
        $charge_amount = OsPaymentsSquareupHelper::convert_charge_amount_to_requirements($charge_amount);
      }
    }
    return $charge_amount;
  }




  public function output_payment_step_contents($booking, $enabled_payment_times){
    if(!OsPaymentsHelper::is_payment_processor_enabled($this->processor_code)) return; ?>
      <div class="lp-payment-method-content" data-payment-method="card" data-init-route="<?php echo OsRouterHelper::build_route_name('payments_squareup', 'create_payment_intent'); ?>">
        <div class="lp-payment-method-content-i">
        <div class="lp-payment-method-card-w">
        <?php if(OsSettingsHelper::is_env_demo()) echo '<div class="lp-demo-mode-msg">'.__('Demo Mode, click Next Step to proceed', 'latepoint-payments-squareup').'</div>'; ?>
        <div class="lp-card-i">
          <div class="lp-squareup-card-chip"><div class="chip-i"></div></div>
          <div class="payment-type-credit-card">
            <h4 class="lp-card-header"><?php _e('Card Details', 'latepoint-payments-squareup'); ?></h4>
            <div class="token"></div>
            <div class="os-row">
              <?php echo OsFormHelper::text_field('payment[name_on_card]', __('Name on card', 'latepoint-payments-squareup'), '', array('class' => 'required'), array('class' => OsPaymentsSquareupHelper::is_zip_code_removed() ? 'os-col-12' : 'os-col-9')); ?>
              <?php 
              if(!OsPaymentsSquareupHelper::is_zip_code_removed()){
                if(OsSettingsHelper::is_env_demo()){
                  echo OsFormHelper::text_field('payment[zip]', __('ZIP', 'latepoint-payments-squareup'), '', array('class' => 'required'), array('class' => 'os-col-3'));
                }else{ ?>
                  <div class="os-col-3">
                    <div class="os-form-group os-form-group-transparent os-form-textfield-group">
                      <div id="latepoint_square_payment_zip" data-placeholder="<?php _e('Postal Code', 'latepoint-payments-squareup'); ?>" class="os-form-control os-framed-field"></div>
                    </div>
                  </div><?php
                }
              } ?>
            </div>
            <div class="os-row">
              <div class="os-col-12">
                <?php if(OsSettingsHelper::is_env_demo()){ ?>
                  <?php echo OsFormHelper::text_field('payment[card_number]', __('Card Number', 'latepoint-payments-squareup'), ''); ?>
                <?php }else{ ?>
                  <div class="os-form-group os-form-group-transparent os-form-textfield-group">
                    <div id="latepoint_square_payment_card_number" data-placeholder="<?php _e('Enter Card Number', 'latepoint-payments-squareup'); ?>" class="os-form-control os-framed-field"></div>
                  </div>
                <?php } ?>
              </div>
            </div>
            <div class="os-row">
              <div class="os-col-6">
              </div>
              <div class="os-col-3">
                <?php if(OsSettingsHelper::is_env_demo()){ ?>
                  <?php echo OsFormHelper::text_field('payment[exp_date]', __('Exp.Date', 'latepoint-payments-squareup'), ''); ?>
                <?php }else{ ?>
                  <div class="os-form-group os-form-group-transparent os-form-textfield-group">
                    <div id="latepoint_square_payment_card_expiration" data-placeholder="<?php _e('Exp.Date', 'latepoint-payments-squareup'); ?>" class="os-form-control os-framed-field"></div>
                  </div>
                <?php } ?>
              </div>
              <div class="os-col-3">
                <?php if(OsSettingsHelper::is_env_demo()){ ?>
                  <?php echo OsFormHelper::text_field('payment[cvc]', __('CVC', 'latepoint-payments-squareup'), ''); ?>
                <?php }else{ ?>
                  <div class="os-form-group os-form-group-transparent os-form-textfield-group">
                    <div id="latepoint_square_payment_card_cvc" data-placeholder="<?php _e('CVC', 'latepoint-payments-squareup'); ?>" class="os-form-control os-framed-field"></div>
                  </div>
                <?php } ?>
              </div>
            </div>
          </div>
          </div>
        </div>
        <div class="latepoint-secured-payments-label"><?php _e('All transactions are secure and encrypted. Credit card information is never stored.', 'latepoint-payments-squareup'); ?></div>
        </div>
      </div><?php
  }


  public function sub_step_for_payment_step($sub_step){
    if(OsPaymentsHelper::is_payment_processor_enabled($this->processor_code)){
      $sub_step = 'payment-method-content';
    }
    return $sub_step;
  }

  public function prepare_booking_object_for_step($booking_object, $step_name){
    if($step_name == 'payment'){
      if(OsPaymentsHelper::is_payment_processor_enabled($this->processor_code)){
      }
    }
    return $booking_object;
  }

  public function add_vars_for_payment_step($vars, $booking_object, $step_name){
    if($step_name == 'payment'){
      if(OsPaymentsHelper::is_payment_processor_enabled($this->processor_code)){

      }
    }
    return $vars;
  }


  public function get_supported_payment_methods(){
    return [
            'card' => [
                        'name' => __('Card', 'latepoint-payments-squareup'), 
                        'label' => __('Card', 'latepoint-payments-squareup'), 
                        'image_url' => LATEPOINT_IMAGES_URL.'payment_cards.png',
                        'code' => 'card',
                        'time_type' => 'now'
                      ]
          ];
  }


  public function register_enabled_payment_methods($enabled_payment_methods){
    if(OsPaymentsHelper::is_payment_processor_enabled($this->processor_code)){
      $enabled_payment_methods = array_merge($enabled_payment_methods, $this->get_supported_payment_methods());
    }
    return $enabled_payment_methods;
  }

  // adds payment method to payment settings
  public function register_payment_methods($payment_methods){
    $payment_methods = array_merge($payment_methods, $this->get_supported_payment_methods());
    return $payment_methods;
  }

  public function register_payment_processor($payment_processors, $enabled_only){
    $payment_processors[$this->processor_code] = ['code' => $this->processor_code, 'name' => __('Squareup', 'latepoint-payments-squareup'), 'image_url' => $this->images_url().'processor-logo.png'];
    return $payment_processors;
  }

  public function add_settings_fields($processor_code){
    if($processor_code != $this->processor_code) return false; ?>
      <h3 class="os-sub-header"><?php _e('API Keys', 'latepoint-payments-squareup'); ?></h3>
      <div class="os-row">
        <div class="os-col-6">
          <?php echo OsFormHelper::text_field('settings[squareup_publishable_key]', __('Application ID', 'latepoint-payments-squareup'), OsSettingsHelper::get_settings_value('squareup_publishable_key')); ?>
        </div>
        <div class="os-col-6">
          <?php echo OsFormHelper::password_field('settings[squareup_secret_key]', __('Access Token', 'latepoint-payments-squareup'), OsSettingsHelper::get_settings_value('squareup_secret_key')); ?>
        </div>
      </div>
      <h3 class="os-sub-header"><?php _e('Other Settings', 'latepoint-payments-squareup'); ?></h3>
      <?php  
      $selected_squareup_country_code = OsSettingsHelper::get_settings_value('squareup_country_code', OsPaymentsSquareupHelper::$default_country_code);
      $country_currencies = OsPaymentsSquareupHelper::load_currencies_list();
      $selected_squareup_currency_iso_code = OsSettingsHelper::get_settings_value('squareup_currency_iso_code', OsPaymentsSquareupHelper::$default_currency_iso_code); ?>
      <div class="os-row">
        <div class="os-col-6">
          <?php echo OsFormHelper::select_field('settings[squareup_country_code]', __('Country', 'latepoint-payments-squareup'), OsPaymentsSquareupHelper::load_countries_list(), $selected_squareup_country_code); ?>
        </div>
        <div class="os-col-6">
          <?php echo OsFormHelper::select_field('settings[squareup_currency_iso_code]', __('Currency Code', 'latepoint-payments-squareup'), $country_currencies, $selected_squareup_currency_iso_code); ?>
        </div>
      </div>
    <?php
  }

  /**
   * Init LatePoint when WordPress Initialises.
   */
  public function init() {
    // Set up localisation.
    $this->load_plugin_textdomain();
  }

  public function latepoint_init(){
    if(OsPaymentsHelper::is_payment_processor_enabled($this->processor_code)) OsPaymentsSquareupHelper::set_api_key();
    LatePoint\Cerber\Router::init_addon();
  }


  public function load_plugin_textdomain() {
    load_plugin_textdomain('latepoint-payments-squareup', false, dirname(plugin_basename(__FILE__)) . '/languages');
  }



  public function on_deactivate(){
  }

  public function on_activate(){
    if(class_exists('OsDatabaseHelper')) OsDatabaseHelper::check_db_version_for_addons();
    do_action('latepoint_on_addon_activate', $this->addon_name, $this->version);
  }

  public function register_addon($installed_addons){
    $installed_addons[] = ['name' => $this->addon_name, 'db_version' => $this->db_version, 'version' => $this->version];
    return $installed_addons;
  }




  public function load_front_scripts_and_styles(){
    if(OsPaymentsHelper::is_payment_processor_enabled($this->processor_code)){
      // Stylesheets
      wp_enqueue_style( 'latepoint-payments-squareup-front', $this->public_stylesheets() . 'latepoint-payments-squareup-front.css', false, $this->version );

      // Javascripts
      $square_js_source = OsSettingsHelper::is_env_payments_live() ? 'https://js.squareup.com/v2/paymentform' : 'https://js.squareupsandbox.com/v2/paymentform';
      wp_enqueue_script( 'squareup', $square_js_source, false, null );
      wp_enqueue_script( 'latepoint-payments-squareup',  $this->public_javascripts() . 'latepoint-payments-squareup.js', array('jquery', 'squareup', 'latepoint-main-front'), $this->version );
    }

  }

  public function load_admin_scripts_and_styles(){
    if(OsPaymentsHelper::is_payment_processor_enabled($this->processor_code)){
      // Stylesheets
      wp_enqueue_style( 'latepoint-payments-squareup-back', $this->public_stylesheets() . 'latepoint-payments-squareup-back.css', false, $this->version );

      // Javascripts
    }
  }


  public function localized_vars_for_admin($localized_vars){
    return $localized_vars;
  }

  public function localized_vars_for_front($localized_vars){
    if(OsPaymentsHelper::is_payment_processor_enabled($this->processor_code)){
      $localized_vars['squareup_key'] = OsPaymentsSquareupHelper::get_publishable_key();
      $localized_vars['is_squareup_active'] = true;
      $localized_vars['squareup_payment_options_route'] = OsRouterHelper::build_route_name('payments_squareup', 'get_payment_options');
    }else{
      $localized_vars['is_squareup_active'] = false;

    }
    return $localized_vars;
  }

}

endif;

if ( in_array( 'latepoint/latepoint.php', get_option( 'active_plugins', array() ) )  || array_key_exists('latepoint/latepoint.php', get_site_option('active_sitewide_plugins', array())) ) {
  $LATEPOINT_ADDON_PAYMENTS_RAZORPAY = new LatePointPaymentsSquareup();
}
$latepoint_session_salt = 'ZjYwNTU0MzgtNDAwNC00OTgzLTgzMTUtNWZmNDlkNmUwNjUy';
