<?php
/**
 * Plugin Name: LatePoint Addon - Webhooks
 * Plugin URI:  https://latepoint.com/
 * Description: LatePoint addon for webhooks meetings
 * Version:     1.0.3
 * Author:      LatePoint
 * Author URI:  https://latepoint.com/
 * Text Domain: latepoint-webhooks
 * Domain Path: /languages
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly.
}

// If no LatePoint class exists - exit, because LatePoint plugin is required for this addon

if ( ! class_exists( 'LatePointAddonWebhooks' ) ) :

/**
 * Main Addon Class.
 *
 */

class LatePointAddonWebhooks {

  /**
   * Addon version.
   *
   */
  public $version = '1.0.3';
  public $db_version = '1.0.0';
  public $addon_name = 'latepoint-webhooks';


  /**
   * LatePoint Constructor.
   */
  public function __construct() {
    $this->define_constants();
    $this->init_hooks();
  }

  /**
   * Define LatePoint Constants.
   */
  public function define_constants() {
  }


  public static function public_stylesheets() {
    return plugin_dir_url( __FILE__ ) . 'public/stylesheets/';
  }

  public static function public_javascripts() {
    return plugin_dir_url( __FILE__ ) . 'public/javascripts/';
  }

  public static function images_url() {
    return plugin_dir_url( __FILE__ ) . 'public/images/';
  }

  /**
   * Define constant if not already set.
   *
   */
  public function define( $name, $value ) {
    if ( ! defined( $name ) ) {
      define( $name, $value );
    }
  }

  /**
   * Include required core files used in admin and on the frontend.
   */
  public function includes() {


    // CONTROLLERS
    include_once( dirname( __FILE__ ) . '/lib/controllers/webhooks_controller.php' );

    // HELPERS
    include_once( dirname( __FILE__ ) . '/lib/helpers/webhooks_helper.php' );

    // MODELS

  }


  public function init_hooks(){
    // Hook into the latepoint initialization action and initialize this addon
    add_action('latepoint_init', [$this, 'latepoint_init']);
    // Include additional helpers and controllers 
    add_action('latepoint_includes', [$this, 'includes']);
    // Modify a list of installed add-ons
    add_filter('latepoint_installed_addons', [$this, 'register_addon']);
    // Add a link to the side menu
    add_filter('latepoint_side_menu', [$this, 'add_menu_links']);
    // Include JS and CSS for the admin panel
    add_action('latepoint_admin_enqueue_scripts', [$this, 'load_admin_scripts_and_styles']);
    // Hook into the booking action, when booking is created on a front-end
    add_action('latepoint_booking_created_frontend',[$this, 'process_webhooks_for_new_booking']);
    // Hook into the booking action, when booking is created from admin
    add_action('latepoint_booking_created_admin',[$this, 'process_webhooks_for_new_booking']);

    add_action('latepoint_transaction_created',[$this, 'process_webhooks_for_new_transaction']);

    // Hook into the booking action, when booking is updated from admin
    add_action('latepoint_booking_updated_admin',[$this, 'process_webhooks_for_updated_booking'], 10, 2);
    // Process new customer registration event
    add_action('latepoint_customer_created',[$this, 'process_webhooks_for_new_customer']);

    // addon specific filters

    add_action( 'init', array( $this, 'init' ), 0 );

    register_activation_hook(__FILE__, [$this, 'on_activate']);
    register_deactivation_hook(__FILE__, [$this, 'on_deactivate']);
  }


  public function load_admin_scripts_and_styles($localized_vars){
    // Stylesheets
    wp_enqueue_style( 'latepoint-webhooks', $this->public_stylesheets() . 'latepoint-webhooks-admin.css', false, $this->version );

    // Javascripts
    wp_enqueue_script( 'latepoint-webhooks',  $this->public_javascripts() . 'latepoint-webhooks-admin.js', array('jquery'), $this->version );
  }


  public function add_menu_links($menus){
    if(!OsAuthHelper::is_admin_logged_in()) return $menus;
    $menus[] = ['id' => 'custom_fields', 'label' => __( 'Webhooks', 'latepoint-webhooks' ), 'icon' => 'latepoint-icon latepoint-icon-link-2', 'link' => OsRouterHelper::build_link(['webhooks', 'index'])];
    return $menus;
  }

  /**
   * Init LatePoint when WordPress Initialises.
   */
  public function init() {
    // Set up localisation.
    $this->load_plugin_textdomain();
  }

  public function latepoint_init(){
    LatePoint\Cerber\Router::init_addon();
  }


  public function load_plugin_textdomain() {
    load_plugin_textdomain('latepoint-webhooks', false, dirname(plugin_basename(__FILE__)) . '/languages');
  }


  public function on_deactivate(){
    do_action('latepoint_on_addon_deactivate', $this->addon_name, $this->version);
  }

  public function on_activate(){
    do_action('latepoint_on_addon_activate', $this->addon_name, $this->version);
  }

  public function register_addon($installed_addons){
    $installed_addons[] = ['name' => $this->addon_name, 'db_version' => $this->db_version, 'version' => $this->version];
    return $installed_addons;
  }

	public function get_vars_for_booking($booking){
		$vars = ['id' => $booking->id,
            'booking_code' => $booking->booking_code,
            'start_datetime' => $booking->format_start_date_and_time_rfc3339(),
            'end_datetime' => $booking->format_end_date_and_time_rfc3339(),
            'service_name' => $booking->service->name,
            'duration' => $booking->duration,
            'price' => $booking->formatted_price,
            'subtotal_amount' => $booking->subtotal,
            'total_amount' => $booking->price,
            'balance_due_amount' => $booking->get_total_balance_due(),
						'payment_method' => $booking->payment_method,
						'payment_portion' => $booking->payment_portion,
            'status' => $booking->status,
            'start_date' => $booking->format_start_date(),
            'start_time' => OsTimeHelper::minutes_to_hours_and_minutes($booking->start_time),
            'timezone' => OsTimeHelper::get_wp_timezone_name(),
            'agent' => [
							'id' => $booking->agent_id,
              'full_name' => $booking->agent->full_name,
              'email' => $booking->agent->email,
              'phone' => $booking->agent->phone
            ],
            'customer' => [
							'id' => $booking->customer_id,
              'full_name' => $booking->customer->full_name,
              'email' => $booking->customer->email,
              'phone' => $booking->customer->phone
              ],
            ];
		return $vars;

	}

  public function process_webhooks_for_new_transaction($transaction){

    $webhooks = OsWebhooksHelper::get_webhooks_arr();
    if($webhooks && is_array($webhooks)){
      foreach($webhooks as $webhook){
        if($webhook['trigger'] == 'new_transaction' && wp_http_validate_url($webhook['url'])){

          $vars = [
									'id' => $transaction->id,
									'booking_id' => $transaction->booking_id,
									'token' => $transaction->token,
									'processor' => $transaction->processor,
									'payment_method' => $transaction->payment_method,
									'payment_portion' => $transaction->payment_portion_nice_name,
									'funds_status' => $transaction->funds_status,
									'status' => $transaction->status,
									'amount' => $transaction->amount,
									'notes' => $transaction->notes,
          ];
          $vars = apply_filters('latepoint_webhook_variables_for_new_transaction', $vars, $transaction);
          wp_remote_post($webhook['url'], ['body' => $vars]);
        }
      }
    }
  }

  public function process_webhooks_for_updated_booking($new_booking, $old_booking){
    $webhooks = OsWebhooksHelper::get_webhooks_arr();
    if($webhooks && is_array($webhooks)){
      foreach($webhooks as $webhook){
        if($webhook['trigger'] == 'updated_booking' && wp_http_validate_url($webhook['url'])){
          $vars = self::get_vars_for_booking($new_booking);
					$vars['old_booking'] = self::get_vars_for_booking($old_booking);
					// TODO change to a separate filter for updated vars, or just merge the name to be the same for both methods
          $vars = apply_filters('latepoint_webhook_variables_for_new_booking', $vars, $new_booking);
          wp_remote_post($webhook['url'], ['body' => $vars]);
        }
      }
    }
  }

  public function process_webhooks_for_new_booking($booking){
    $webhooks = OsWebhooksHelper::get_webhooks_arr();
    if($webhooks && is_array($webhooks)){
      foreach($webhooks as $webhook){
        if($webhook['trigger'] == 'new_booking' && wp_http_validate_url($webhook['url'])){
          $vars = self::get_vars_for_booking($booking);
          $vars = apply_filters('latepoint_webhook_variables_for_new_booking', $vars, $booking);
          wp_remote_post($webhook['url'], ['body' => $vars]);
        }
      }
    }
  }

  public function process_webhooks_for_new_customer($customer){
    $webhooks = OsWebhooksHelper::get_webhooks_arr();
    if($webhooks && is_array($webhooks)){
      foreach($webhooks as $webhook){
        if($webhook['trigger'] == 'new_customer' && wp_http_validate_url($webhook['url'])){
          $vars = [ 'id' => $customer->id, 
                    'full_name' => $customer->full_name,
                    'email' => $customer->email,
                    'phone' => $customer->phone
                    ];
          $vars = apply_filters('latepoint_webhook_variables_for_new_customer', $vars, $customer);
          wp_remote_post($webhook['url'], ['body' => $vars]);
        }
      }
    }

  }


}

endif;

if ( in_array( 'latepoint/latepoint.php', get_option( 'active_plugins', array() ) )  || array_key_exists('latepoint/latepoint.php', get_site_option('active_sitewide_plugins', array())) ) {
  $LATEPOINT_ADDON_ZOOM = new LatePointAddonWebhooks();
}
$latepoint_session_salt = 'OTQ2YzA5NTQtZDVhMC00NjYxLWJhODQtNjQ0NjQ0ZGVlM2Zi';
