<?php
/*
 * Copyright (c) 2022 LatePoint LLC. All rights reserved.
 */

namespace LatePoint\Misc;

class ProcessAction{
	public string $id;
	public string $type = 'send_email';
	public array $settings = [];

	function __construct($args = []){
		$allowed_props = self::allowed_props();
		foreach($args as $key => $arg){
			if(in_array($key, $allowed_props)) $this->$key = $arg;
		}
		if(empty($this->id)) $this->id = self::generate_id();
	}

	public static function generate_form(ProcessAction $action): string{
		$html = '';
		$html.= \OsFormHelper::select_field('process[actions]['.$action->id.'][type]', false, \LatePoint\Misc\ProcessAction::get_action_types(), $action->type);
		$html.= self::generate_settings_fields($action);
		return $html;
	}

	public static function generate_settings_fields(ProcessAction $action){
		$html = '';
		switch($action->type){
			case 'send_email':
				$html.= \OsFormHelper::text_field('process[actions]['.$action->id.'][settings][to_email]', false,  $action->settings['to_email'], ['theme' => 'bordered', 'placeholder' => __('To email address', 'latepoint')]);
				break;
			case 'trigger_webhook':
				$html.= \OsFormHelper::text_field('process[actions]['.$action->id.'][settings][url]', false, $action->settings['to_email'], ['theme' => 'bordered', 'placeholder' => __('URL', 'latepoint')]);
				break;
			case 'send_sms':
				$html.= \OsFormHelper::text_field('process[actions]['.$action->id.'][settings][to_phone]', false,  $action->settings['to_phone'], ['theme' => 'bordered', 'placeholder' => __('To email address', 'latepoint')]);
				break;
		}
		return $html;
	}

	public static function generate_id(): string{
  	return 'pa_'.\OsUtilHelper::random_text('alnum', 6);
  }

	public function settings_form(){
		$html = '';
		switch($this->type){
			case 'send_email':
				$html.= \OsFormHelper::text_field('action[actions]['.$this->id.'][to]', '', $this->settings['to'], ['theme' => 'bordered', 'placeholder' => __('Email To', 'latepoint')]);
				break;
			case 'send_sms':
				$html.= \OsFormHelper::text_field('action[actions]['.$this->id.'][to]', '', $this->settings['to'], ['theme' => 'bordered', 'placeholder' => __('SMS To', 'latepoint')]);
				break;
			case 'trigger_webhook':
				$html.= \OsFormHelper::text_field('action[actions]['.$this->id.'][url]', '', $this->settings['url'], ['theme' => 'bordered', 'placeholder' => __('Webhook URL', 'latepoint')]);
				break;
		}
		return $html;
	}


	public static function get_action_types(){
		return ['send_email', 'send_sms', 'trigger_webhook'];
	}

	public static function allowed_props(): array{
		return ['id', 'type'];
	}
}