<?php
/*
 * Copyright (c) 2022 LatePoint LLC. All rights reserved.
 */

namespace LatePoint\Misc;

class ProcessEvent{
	public string $type = 'booking_created';
	public array $conditions = [];

	function __construct($args = []){
		$allowed_props = self::allowed_props();
		foreach($args as $key => $arg){
			if(in_array($key, $allowed_props)) $this->$key = $arg;
		}
	}

	public function conditions_form_html(){
		$html = '';
		if($this->conditions){
			foreach($this->conditions as $condition){
				$html.= $this->generate_condition_form_html($condition);
			}
		}else{
			$html.= $this->generate_condition_form_html();
		}
		return $html;
	}

	public function generate_condition_form_html($condition = false){
    $properties = self::condition_properties_for_type($this->type);
    $properties = apply_filters('latepoint_process_event_condition_properties', $properties);
    // new condition
    if(!$condition){
      $condition = [
				'id' => self::generate_condition_id(),
	      'property' => '',
	      'operator' => false,
	      'value' => false
      ];
    }
    $html = '<div class="cf-condition" data-condition-id="'.$condition['id'].'">'.
                '<button class="cf-remove-condition"><i class="latepoint-icon latepoint-icon-cross"></i></button>'.
                \OsFormHelper::select_field('event[conditions]['.$condition['id'].'][property]', false, $properties, $condition['property'], ['class' => 'custom-field-condition-property', 'data-route' => \OsRouterHelper::build_route_name('custom_fields', 'available_values_for_condition_property')], ['class' => 'custom-field-condition-property-w']).
                \OsFormHelper::select_field('event[conditions]['.$condition['id'].'][operator]', false, array('equal' => __('is equal to', 'latepoint'), 'not_equal' => __('is not equal to', 'latepoint')), $condition['operator'], [], ['class' => 'custom-field-condition-operator-w']).
                \OsFormHelper::multi_select_field('event[conditions]['.$condition['id'].'][value]', false, \OsFormHelper::model_options_for_multi_select($condition['property']), $condition['value'] ? explode(',', $condition['value']) : [], [], ['class' => 'custom-field-condition-values-w']).
                '<div class="custom-field-condition-property-w" 
                      data-os-action="'.\OsRouterHelper::build_route_name('custom_fields', 'new_condition').'" 
                      data-os-params="'.\OsUtilHelper::build_os_params(['custom_field_id' => $custom_field_id]).'" 
                      data-os-pass-response="yes" 
                      data-os-pass-this="yes" 
                      data-os-before-after="none" 
                      data-os-after-call="latepointCustomFieldsAdminAddon.add_custom_field_condition"><button class="latepoint-btn-outline latepoint-btn"><i class="latepoint-icon latepoint-icon-plus2"></i><span>'.__('AND', 'latepoint').'</span></button></div>'.
            '</div>';
    return $html;
	}

	public static function condition_properties_for_type($event_type){
		switch ($event_type){
			case 'booking_created':
				$properties = [ 'booking_status' => __('Booking Status', 'latepoint'),
					'service' => __('Service', 'latepoint'),
					'agent' => __('Agent', 'latepoint')];
				break;
		}
		return $properties;
	}

	public static function get_event_types(){
		return ['booking_created', 'booking_updated', 'customer_created', 'customer_updated', 'transaction_created'];
	}

	public static function generate_condition_id(): string{
  	return 'pec_'.\OsUtilHelper::random_text('alnum', 6);
  }

	public static function allowed_props(): array{
		return ['id', 'type', 'conditions'];
	}
}